@@ -38,6 +38,7 @@ struct CrackleConfig {
 enum Focus {
     MessageInput,
     JoinRoomInput,
+    InitialSync,
     //RoomList,
     //PaticipantList,
     //History,
@@ -48,6 +49,25 @@ enum Focus {
     RoomSelect,
 }
 
+fn find_top(bottom_index: usize, height: usize, num_rooms: usize) -> usize {
+    if num_rooms < height - 1 {
+        return 0;
+    } else {
+        return bottom_index - (height - 3);
+    }
+}
+
+fn find_bottom(top_index: usize, height: usize, num_rooms: usize) -> usize {
+    if top_index + (height - 3) >= num_rooms {
+        if num_rooms == 0 {
+            return 0;
+        } else {
+            return num_rooms - 1;
+        }
+    } else {
+        return top_index + (height - 3);
+    }
+}
 /// App holds the state of the application
 struct App {
     /// Current value of the input box
@@ -68,6 +88,10 @@ struct App {
     auth_token: Option<String>,
     joined_rooms: Vec<RoomId>,
     room_to_direct_name: BTreeMap<RoomId, String>,
+    room_list_top: usize,
+    room_list_height: usize,
+    room_list_top_backup: usize,
+    room_list_prev_height: usize,
 }
 
 impl App {
@@ -97,7 +121,7 @@ impl App {
         };
         Self {
             input: String::new(),
-            focus: Focus::Watching,
+            focus: Focus::InitialSync,
             messages: BTreeMap::new(),
             host: hs_url,
             server_name: server_name,
@@ -111,6 +135,10 @@ impl App {
             auth_token: auth_token,
             room: room,
             room_to_direct_name: BTreeMap::new(),
+            room_list_top: 0,
+            room_list_height: 2,
+            room_list_top_backup: 0,
+            room_list_prev_height: 2,
         }
     }
 }
@@ -190,11 +218,12 @@ async fn sync(client: ruma_client::Client::<ruma_client::http_client::Reqwest>,
     sync_result.unwrap()
 }
 */
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Eq, PartialEq)]
 enum SyncMessage {
     Quit,
     //Response(sync::sync_events::v3::Response),
     SendMessage(RoomId, String, String),
+    InitialSyncOver,
     RoomsUpdated(
         Vec<RoomId>,
         BTreeMap<RoomId, Box<RoomName>>,
@@ -228,6 +257,7 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
         let mut sync_results = client.sync().unwrap();
         //let mut known_room_list: Vec<RoomId> = Vec::new();
         client.extract_data(sync_results.clone());
+        tx1.send(SyncMessage::InitialSyncOver).unwrap();
         //let mut f = std::fs::File::create("sync.log").unwrap();
         //writeln!(f, "{:#?}", sync_results);
         //client.join_with_alias(app.room);
@@ -303,12 +333,22 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                                         if index_num == 0 {
                                             if first.clone() == app.selected_room.clone().unwrap() {
                                                 app.selected_room = Some(last.clone());
+                                                app.room_list_top = find_top(
+                                                    app.joined_rooms.len() - 1,
+                                                    app.room_list_height,
+                                                    app.joined_rooms.len(),
+                                                );
                                                 break;
                                             }
                                             prev = room_id.clone();
                                         } else {
                                             if room_id.clone() == app.selected_room.clone().unwrap()
                                             {
+                                                if room_id.clone()
+                                                    == app.joined_rooms[app.room_list_top]
+                                                {
+                                                    app.room_list_top -= 1;
+                                                }
                                                 app.selected_room = Some(prev);
                                                 break;
                                             }
@@ -326,22 +366,25 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                                     let mut use_next = false;
                                     for room_id in app.joined_rooms.iter() {
                                         if use_next {
+                                            if app.selected_room.clone().unwrap()
+                                                == app.joined_rooms[find_bottom(
+                                                    app.room_list_top,
+                                                    app.room_list_height,
+                                                    app.joined_rooms.len(),
+                                                )]
+                                            {
+                                                app.room_list_top += 1;
+                                            }
                                             app.selected_room = Some(room_id.clone());
                                             break;
-                                        }
-                                        /*else if index_num == 0 {
-                                            //first = room_id.clone();
-                                            if first.clone() == app.selected_room.clone().unwrap() {
-                                                use_next = true;
-                                            }
-                                        }*/
-                                        else if room_id != last {
+                                        } else if room_id != last {
                                             if room_id.clone() == app.selected_room.clone().unwrap()
                                             {
                                                 use_next = true;
                                             }
                                         } else {
                                             app.selected_room = Some(first.clone());
+                                            app.room_list_top = 0;
                                         }
                                     }
                                 }
@@ -362,10 +405,12 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                                 app.current_room = Some(app.selected_room.clone().unwrap());
                                 app.selected_room = None;
                                 app.focus = Focus::Watching;
+                                app.room_list_top_backup = app.room_list_top;
                             }
                             KeyCode::Esc => {
                                 app.selected_room = None;
                                 app.focus = Focus::Watching;
+                                app.room_list_top = app.room_list_top_backup;
                             }
                             _ => {}
                         }
@@ -454,17 +499,20 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                             */
                         }
                     },
+                    Focus::InitialSync => {}
                 }
             }
         } else {
             if let Ok(msg) = rx1.try_recv() {
                 if let SyncMessage::SendMessage(room_id, user_id, message) = msg {
                     let mut msgs = app.messages[&room_id].clone();
+                    msgs.push(format!("<{}> {}", user_id, message));
                     if msgs.len() > 100 {
                         msgs.remove(0);
                     }
-                    msgs.push(format!("<{}> {}", user_id, message));
                     app.messages.insert(room_id.clone(), msgs);
+                } else if msg == SyncMessage::InitialSyncOver {
+                    app.focus = Focus::Watching;
                 } else if let SyncMessage::RoomsUpdated(
                     room_id_list,
                     room_to_name,
@@ -517,6 +565,8 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
         .direction(Direction::Horizontal)
         .constraints([Constraint::Percentage(20), Constraint::Percentage(80)].as_ref())
         .split(chunks[1]);
+    app.room_list_prev_height = app.room_list_height;
+    app.room_list_height = middle_chunk[0].height as usize;
     let (msg, style) = match app.focus {
         Focus::Watching => (
             vec![
@@ -560,6 +610,12 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
             ],
             Style::default(),
         ),
+        Focus::InitialSync => (
+            vec![
+                Span::raw("Performing inital sync.  Please wait..."),
+            ],
+            Style::default().add_modifier(Modifier::RAPID_BLINK),
+        ),
     };
     let mut text = Text::from(Spans::from(msg));
     text.patch_style(style);
@@ -572,13 +628,10 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
             Focus::MessageInput => Style::default().fg(Color::Yellow),
             Focus::RoomSelect => Style::default(),
             Focus::JoinRoomInput => Style::default().fg(Color::Yellow),
+            Focus::InitialSync => Style::default(),
         })
         .block(Block::default().borders(Borders::ALL).title("Input"));
     match app.focus {
-        Focus::Watching =>
-            // Hide the cursor. `Frame` does this by default, so we don't need to do anything here
-            {}
-
         Focus::MessageInput => {
             // Make the cursor visible and ask tui-rs to put it at the specified coordinates after rendering
             f.set_cursor(
@@ -597,14 +650,69 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
                 chunks[2].y + 1,
             )
         }
-        Focus::RoomSelect => {}
+        _ => {}
     }
     //println!("{:#?}", app.joined_rooms);
+    if app.joined_rooms.len() > 0 {
+        if app.room_list_prev_height != app.room_list_height {
+            //let location: bool;
+            let chosen: RoomId;
+            if app.selected_room.is_none() {
+                chosen = app.current_room.clone().unwrap();
+            } else {
+                chosen = app.selected_room.clone().unwrap();
+            }
+            let chosen_index = app.joined_rooms.iter().position(|y| &chosen == y).unwrap();
+            if chosen_index
+                > find_bottom(
+                    app.room_list_top,
+                    app.room_list_height,
+                    app.joined_rooms.len(),
+                )
+            {
+                app.room_list_top = chosen_index;
+            } else if chosen_index < app.room_list_top {
+                app.room_list_top =
+                    find_top(chosen_index, app.room_list_height, app.joined_rooms.len());
+            } else {
+                app.room_list_top = find_top(
+                    find_bottom(
+                        app.room_list_top,
+                        app.room_list_height,
+                        app.joined_rooms.len(),
+                    ),
+                    app.room_list_height,
+                    app.joined_rooms.len(),
+                );
+            }
+        } /*else if app.room_list_prev_height > app.room_list_height {
+              app.room_list_top = find_top(
+                  find_bottom(
+                      app.room_list_top - (app.room_list_prev_height - app.room_list_height),
+                      app.room_list_height,
+                      app.joined_rooms.len(),
+                  ),
+                  app.room_list_height,
+                  app.joined_rooms.len(),
+              );
+              //app.room_list_top -= app.room_list_height - app.room_list_prev_height;
+          }*/
+    }
     let room_list: Vec<ListItem> = app
         .joined_rooms
         .iter()
-        //.enumerate()
-        .map(|room_id| {
+        .enumerate()
+        .filter_map(|(index_num, room_id)| {
+            if index_num < app.room_list_top
+                || index_num
+                    > find_bottom(
+                        app.room_list_top,
+                        app.room_list_height,
+                        app.joined_rooms.len(),
+                    )
+            {
+                return None;
+            }
             let content: Vec<Spans>;
             if app.rooms.contains_key(&room_id) {
                 content = vec![Spans::from(Span::raw(format!("{}", app.rooms[room_id])))];
@@ -646,6 +754,33 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
                 Focus::RoomSelect => {
                     if app.selected_room.is_none() {
                         app.selected_room = Some(current_room);
+                    } else if app
+                        .joined_rooms
+                        .iter()
+                        .position(|x| x.clone() == app.selected_room.clone().unwrap())
+                        .unwrap()
+                        < app.room_list_top
+                    {
+                        app.selected_room = Some(app.joined_rooms[app.room_list_top].clone());
+                    } else if app
+                        .joined_rooms
+                        .iter()
+                        .position(|x| x.clone() == app.selected_room.clone().unwrap())
+                        .unwrap()
+                        > find_bottom(
+                            app.room_list_top,
+                            app.room_list_height,
+                            app.joined_rooms.len(),
+                        )
+                    {
+                        app.selected_room = Some(
+                            app.joined_rooms[find_bottom(
+                                app.room_list_top,
+                                app.room_list_height,
+                                app.joined_rooms.len(),
+                            )]
+                            .clone(),
+                        );
                     }
                     if app.selected_room.clone().unwrap() == room_id.clone() {
                         z = z.style(style.add_modifier(Modifier::REVERSED));
@@ -653,7 +788,7 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
                 }
                 _ => {}
             }
-            z
+            Some(z)
         })
         .collect();
     let room_list =
@@ -662,13 +797,10 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
     if app.current_room.is_some() {
         let ms = app.messages[&app.current_room.clone().unwrap()].join("\n");
         //let width: i32 = middle_chunk[1].width.try_into().unwrap(); //message_box_height.load().try_into().unwrap();
-        let lines: Vec<String> = textwrap::wrap(
-            ms.as_str(),
-            textwrap::Options::new((middle_chunk[1].width - 2).into()),
-        )
-        .iter()
-        .map(|x| x.to_string())
-        .collect();
+        let lines: Vec<String> = textwrap::wrap(ms.as_str(), (middle_chunk[1].width - 2) as usize)
+            .iter()
+            .map(|x| x.to_string())
+            .collect();
         //.split("\n")
         //.map(|x| x.to_string())
         //.collect();
